#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <unistd.h>


main(int argc, char* argv[])
{
	char *chunk = NULL;
	int fd = -1;
	size_t page_size = getpagesize();
	size_t file_size = 0;
	size_t num_pages = 0;
	size_t current_page = 0;
	size_t buffer_size = 0;
	size_t i = 0;
	unsigned char crc = 0;

	if(argc != 2)
	{
		printf("Usage: es3 FileName\n");
		return;
	}

	fd = open(argv[1], O_RDONLY);
	if(fd < 0)
	{
		perror("open failed");
		return;
	}
	
	file_size = lseek(fd, 0, SEEK_END);
	lseek(fd, 0, SEEK_SET);
	
	num_pages = file_size / page_size;
	if(file_size % page_size)
		++num_pages;
	
	for(current_page = 0; current_page < num_pages; current_page++)
	{
		buffer_size = (current_page == (num_pages - 1)) ? file_size % page_size : page_size;
		
		chunk = mmap(0, buffer_size, PROT_READ, MAP_PRIVATE, fd, page_size * current_page);
		if(chunk == MAP_FAILED)
		{
			perror("map failed");
			close(fd);
			return;
		}
	
		for(i = 0; i < buffer_size; ++i)
			crc ^= chunk[i];

		munmap(chunk, buffer_size);
	}

	close(fd);

	printf("crc = %x\n", crc);
}
