#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>

#define N_BUF		10
#define BUF_SIZE	4

struct DataBuffer
{
	size_t	ReadCount;
	char	Buffer[BUF_SIZE];
};

int sem_create()
{
	int ev;
 
	ev = semget(IPC_PRIVATE, 1, IPC_CREAT | 0666);

	return ev;
}

int sem_set(int ev, int newVal)
{
	static struct sembuf op_set[1] = { {0, 0, 0} };
	op_set[0].sem_op = newVal;

	return (semop(ev, op_set, 1) >=  0);
}

int sem_wait(int ev)
{
	static struct sembuf op_wait[1] = { {0, -1, 0} };
	return (semop(ev, op_wait, 1) >= 0);
}

int sem_release(int ev)
{
	static struct sembuf op_release[1] = { {0, 1, 0} };
	return (semop(ev, op_release, 1) >=  0);
}

int sem_delete(int ev)
{
	return (semctl(ev, 0, IPC_RMID, 0) >= 0);
}

main(int argc, char* argv[])
{
	int sem_writer_resources = sem_create();
	int sem_reader_resources = sem_create();
	int shm_id;
	int fd;
	int i;
	struct DataBuffer *b;
	
	if(argc != 3)
	{
		printf("Usage: cpm SrcFile DstFile\n");
		return;
	}

	shm_id = shmget(IPC_PRIVATE, sizeof(struct DataBuffer) * N_BUF, IPC_CREAT | 0666);

	if(fork() == 0)
	{
		fd = creat(argv[2], 0664);
		b = shmat(shm_id, NULL, SHM_RDONLY);
		i = 0;
		
		for(;;)
		{
			sem_wait(sem_writer_resources);
			if(b[i].ReadCount <= 0)
			{
				break;
			}
			write(fd, b[i].Buffer, b[i].ReadCount);
			sem_release(sem_reader_resources);

			if(++i == N_BUF)
				i = 0;
		}

		shmdt(b);
		close(fd);

		return;
	}

	sem_set(sem_reader_resources, N_BUF);
	
	b = shmat(shm_id, NULL, 0);

	fd = open(argv[1], O_RDONLY);

	i = 0;

	for(;;)
	{
		sem_wait(sem_reader_resources);
		b[i].ReadCount = read(fd, b[i].Buffer, BUF_SIZE);
		sem_release(sem_writer_resources);
		if(b[i].ReadCount <= 0)
		{
			break;
		}
		if(++i == N_BUF)
			i = 0;
	}

	close(fd);
	shmdt(b);

	wait(NULL);

	shmctl(shm_id, IPC_RMID, NULL);
	sem_delete(sem_writer_resources);
	sem_delete(sem_reader_resources);
}

