#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <unistd.h>


main(int argc, char* argv[])
{
	char *chunk_in = NULL, *chunk_out = NULL;
	int fdin = -1;
	int fdout = -1;
	size_t page_size = getpagesize();
	size_t file_size = 0;
	size_t num_pages = 0;
	size_t current_page = 0;
	size_t buffer_size = 0;
	size_t i = 0;
	
	if(argc != 3)
	{
		printf("Usage: %s SrcFile DstFile\n", argv[0]);
		return;
	}

	if(strcmp(argv[1], argv[2]) == 0)
	{
		printf("cannot copy file %s on itself\n", argv[1]);
		return;
	}

	fdin = open(argv[1], O_RDONLY);
	if(fdin < 0)
	{
		perror("open failed");
		return;
	}

	fdout = open(argv[2], O_RDWR | O_CREAT | O_TRUNC, 0600);
	if(fdout < 0)
	{
		perror("open failed");
		return;
	}
	
	file_size = lseek(fdin, 0, SEEK_END);
	if(file_size == 0)
	{
		close(fdin);
		close(fdout);
		return;
	}

	lseek(fdout, file_size - 1, SEEK_SET);
	write(fdout, "\0", 1);
	
	num_pages = file_size / page_size;
	if(file_size % page_size)
		++num_pages;
	
	for(current_page = 0; current_page < num_pages; current_page++)
	{
		buffer_size = (current_page == (num_pages - 1)) ? file_size % page_size : page_size;
		
		chunk_in = mmap(0, buffer_size, PROT_READ, MAP_PRIVATE, fdin, page_size * current_page);
		if(chunk_in == MAP_FAILED)
		{
			perror("map failed");
			close(fdout);
			close(fdin);
			unlink(argv[2]);
			return;
		}

		chunk_out = mmap(0, buffer_size, PROT_WRITE, MAP_SHARED, fdout, page_size * current_page);
		if(chunk_out == MAP_FAILED)
		{
			perror("map failed");
			close(fdout);
			close(fdin);
			unlink(argv[2]);
			return;
		}
	
		memcpy(chunk_out, chunk_in, buffer_size);

		msync(chunk_out, buffer_size, MS_SYNC);

		munmap(chunk_in, buffer_size);
		munmap(chunk_out, buffer_size);
	}

	close(fdin);
	close(fdout);
}
