#include <stdio.h>
#include <unistd.h>

#define MAX_LINE_LENGTH	1024

int OpenFiles(int argc, char* argv[], FILE **fin, FILE **fout)
{
	int opt;

	if(argc == 1)
	{
		*fin = stdin;
		*fout = stdout;
		return 1;
	}

	opterr = 0;
	for(;;)
	{
		opt = getopt(argc, argv, "o:i:");
		if(opt == -1)
		{
			if(optind < argc)
				break;

			if((*fin != NULL) && (*fout != NULL))
				return 1;

			break;
		}
		switch(opt)
		{
		case 'i':
			if(*fin != NULL)
			{
				fprintf(stderr, "Option -i may appear only once in command line\n");
				return 0;
			}

			*fin = fopen(optarg, "r");
			if(*fin == NULL)
			{
				fprintf(stderr, "Cannot open file %s for reading\n", optarg);
				return 0;
			}
			break;
		case 'o':
			if(*fout != NULL)
			{
				fprintf(stderr, "Option -o may appear only once in command line\n");
				return 0;
			}
			
			*fout = fopen(optarg, "w");
			if(*fout == NULL)
			{
				fprintf(stderr, "Cannot open file %s for writing\n", optarg);
				return 0;
			}
			break;
		case '?':
			fprintf(stderr, "Invalid option -%c\n", optopt);
			fprintf(stderr, "Usage: %s -i in_file -o out_file\n", argv[0]);
			return 0;

		default:
			fprintf(stderr, "Usage: %s -i in_file -o out_file\n", argv[0]);
			return 0;
		}
	}
	fprintf(stderr, "Usage: %s -i in_file -o out_file\n", argv[0]);
	return 0;
}

int main(int argc, char* argv[])
{
	FILE *fin = NULL;
	FILE *fout = NULL;
	char line[MAX_LINE_LENGTH];
	int i;

	if(!OpenFiles(argc, argv, &fin, &fout))
	{
		return -1;
	}

	while(fgets(line, sizeof(line) - 1, fin) != NULL)
	{
		i = 0;
		while(line[i])
		{
			if(line[i] >= 'a' && line[i] <= 'z')
				line[i] += 'A' - 'a';
			i++;
		}
		if(fputs(line, fout) < 0)
		{
			fprintf(stderr, "Error writing to file\n");
			return -2;
		}
	}

	if(ferror(fin))
	{
		fprintf(stderr, "Error reading from file\n");
		return -3;
	}

	if(fin != stdin)
		fclose(fin);
	
	if(fout != stdout)
		fclose(fout);

	return 0;
}
